@@ -26,6 +26,9 @@
  */
 
 /* revision history:
+ * 2.1.0a   2005/03/10   hof: added active_layer_tracking feature
+ * 2.1.0a   2004/12/04   hof: added gap_lib_shorten_filename
+ * 2.1.0a   2004/04/18   hof: added gap_lib_fprintf_gdouble
  * 1.3.26a  2004/02/01   hof: added: gap_lib_alloc_ainfo_from_name
  * 1.3.25a  2004/01/20   hof: - removed xvpics support (gap_lib_get_video_paste_name)
  * 1.3.21e  2003/11/04   hof: - gimprc
@@ -113,10 +116,6 @@
 #  endif
 #endif
 
-#ifdef G_OS_WIN32
-#include <direct.h>		/* For _mkdir() */
-#define mkdir(path,mode) _mkdir(path)
-#endif
 
 #ifdef G_OS_WIN32
 #include <process.h>		/* For _getpid() */
@@ -140,12 +139,450 @@ extern      int gap_debug; /* ==0  ... dont print debug infos */
 /* forward  working procedures */
 /* ------------------------------------------ */
 
+gboolean            p_set_active_layer_by_pos(gint32 image_id
+                         ,gint32 ref_layer_stackpos
+                         );
+gboolean            p_set_active_layer_by_name(gint32 image_id
+                          ,gchar *ref_layer_name
+                          ,gint32 ref_layer_stackpos
+                          );
+static gchar *      p_get_active_layer_name(gint32 image_id
+                       ,gint32 *active_layer
+                       ,gint32 *stack_pos
+                       );
+static void         p_do_active_layer_tracking(gint32 image_id
+                       ,GapVinVideoInfo *vin_ptr
+                       ,gchar *ref_layer_name
+                       ,gint32 ref_layer_stackpos
+                       );
+
 static int          p_save_old_frame(GapAnimInfo *ainfo_ptr, GapVinVideoInfo *vin_ptr);
 static int          p_decide_save_as(gint32 image_id, const char *sav_name, const char *final_sav_name);
 static gint32       p_lib_save_named_image2(gint32 image_id, const char *sav_name, GimpRunMode run_mode, gboolean enable_thumbnailsave);
 static char*        p_gzip (char *orig_name, char *new_name, char *zip);
 
 
+
+
+/* ---------------------------------
+ * p_set_active_layer_by_pos
+ * ---------------------------------
+ * set the layer with same stackposition as ref_layer_stackpos
+ * as the active layer, where stackpositions of onionskin layers are not counted.
+ * if the image has less layers than ref_layer_stackpos
+ * then pick the top-most layer, that is no onionskin layer
+ */
+gboolean
+p_set_active_layer_by_pos(gint32 image_id
+                         ,gint32 ref_layer_stackpos
+                         )
+{
+  gint32     *l_layers_list;
+  gint        l_nlayers;
+  gint        l_idx;
+  gboolean    l_is_onion;
+  gint32      l_layer_id;
+  gint32      l_matching_layer_id;
+  gint        l_pos;
+
+  if(gap_debug)
+  {
+    printf("p_set_active_layer_by_pos START\n");
+  }
+  l_pos = 0;
+  l_matching_layer_id = -1;
+  l_layers_list = gimp_image_get_layers(image_id, &l_nlayers);
+  if(l_layers_list)
+  {
+    for(l_idx=0;l_idx < l_nlayers;l_idx++)
+    {
+      l_layer_id = l_layers_list[l_idx];
+      l_is_onion = gap_onion_base_check_is_onion_layer(l_layer_id);
+
+      if(!l_is_onion)
+      {
+        l_matching_layer_id = l_layer_id;
+        if(l_pos == ref_layer_stackpos)
+        {
+          break;
+        }
+        l_pos++;
+      }
+    }
+    
+    g_free(l_layers_list);
+  }
+  
+  if(l_matching_layer_id >= 0)
+  {
+    gimp_image_set_active_layer(image_id, l_matching_layer_id);
+
+    if(gap_debug)
+    {
+      char       *l_name;
+     
+      l_name = gimp_drawable_get_name(l_matching_layer_id);
+      printf("p_set_active_layer_by_pos SET layer_id %d '%s' as ACTIVE\n"
+            , (int)l_matching_layer_id
+	    , l_name
+	    );
+      if(l_name)
+      {
+        g_free(l_name);
+      }	    
+    }
+    return (TRUE);
+  }
+  
+  return (FALSE);
+  
+}  /* end p_set_active_layer_by_pos */
+
+
+
+/* ---------------------------------
+ * p_set_active_layer_by_name
+ * ---------------------------------
+ * set active layer by picking the layer
+ * whos name matches best with ref_layer_name.
+ * ref_layer_stackpos is the 2nd criterium
+ */
+gboolean
+p_set_active_layer_by_name(gint32 image_id
+                          ,gchar *ref_layer_name
+                          ,gint32 ref_layer_stackpos
+                          )
+{
+  gint32     *l_layers_list;
+  gint        l_nlayers;
+  gint        l_idx;
+  gboolean    l_is_onion;
+  gint32      l_layer_id;
+  gint32      l_matching_layer_id;
+  gint        l_pos;
+  gint        l_score;
+  gint        l_case_bonus;
+  gint        l_max_score;
+  gint        l_ref_len;
+  gint        l_len;
+  char       *l_layer_name;
+
+  if(gap_debug)
+  {
+    printf("p_set_active_layer_by_name START\n");
+  }
+  l_pos = 0;
+  l_max_score = 0;
+  l_matching_layer_id = -1;
+  
+  if(ref_layer_name == NULL)
+  {
+    return(p_set_active_layer_by_pos(image_id, ref_layer_stackpos));
+  }
+  
+  l_ref_len = strlen(ref_layer_name);
+  l_layers_list = gimp_image_get_layers(image_id, &l_nlayers);
+  if(l_layers_list)
+  {
+    for(l_idx=0;l_idx < l_nlayers;l_idx++)
+    {
+      l_score = 0;
+      l_case_bonus = 0;
+      l_layer_id = l_layers_list[l_idx];
+      l_layer_name = gimp_drawable_get_name(l_layer_id);
+      if(l_layer_name)
+      {
+        gint ii;
+
+	/* check how many characters of the name are equal */
+        l_len = strlen(l_layer_name);
+        for(ii=0; ii < MIN(l_len, l_ref_len); ii++)
+	{
+	  if(g_ascii_toupper(l_layer_name[ii]) == g_ascii_toupper(ref_layer_name[ii]))
+	  {
+	    /* best score for matching character */
+	    l_score += 8;
+	    if(l_layer_name[ii] == ref_layer_name[ii])
+	    {
+	      if(ii==0)
+	      {
+	        /* prepare bonus for matching in case sensitivity */
+	        l_case_bonus = 4;
+	      }
+	    }
+	    else
+	    {
+	      /* lost the bonus for exact matching in case sensitivity */
+	      l_case_bonus = 0;
+	    }
+	  }
+	  else
+	  {
+            break;
+	  }
+	}
+	
+	l_score += l_case_bonus;
+	if(l_len == l_ref_len)
+	{
+	  /* extra score for equal length */
+	  l_score += 2;
+	}
+	g_free(l_layer_name);
+      }
+      
+      if(l_score == l_max_score)
+      {
+	/* stackposition is checked as secondary criterium
+	 * (score +1) in case there are more names matching
+	 * in the same number of characters
+	 */
+	l_is_onion = gap_onion_base_check_is_onion_layer(l_layer_id);
+	if(!l_is_onion)
+	{
+          if(l_pos == ref_layer_stackpos)
+          {
+            l_score += 1;
+          }
+          l_pos++;
+	}
+      }
+      if(l_score > l_max_score)
+      {
+        l_matching_layer_id = l_layer_id;
+	l_max_score = l_score;
+      }
+    }
+    
+    g_free(l_layers_list);
+  }
+  
+  if(l_matching_layer_id >= 0)
+  {
+    gimp_image_set_active_layer(image_id, l_matching_layer_id);
+
+    if(gap_debug)
+    {
+      char       *l_name;
+     
+      l_name = gimp_drawable_get_name(l_matching_layer_id);
+      printf("p_set_active_layer_by_name SET layer_id %d '%s' as ACTIVE\n"
+            , (int)l_matching_layer_id
+	    , l_name
+	    );
+      if(l_name)
+      {
+        g_free(l_name);
+      }	    
+    }
+    return (TRUE);
+  }
+  
+  return (FALSE);
+}  /* end p_set_active_layer_by_name */
+
+
+/* ---------------------------------
+ * p_get_active_layer_name
+ * ---------------------------------
+ * get id, name and stackposition of the
+ * active_layer in the specified image.
+ * (stackpostitions are counted without onionskin layers
+ *  stackposition 0 is the background layer)
+ */
+gchar *
+p_get_active_layer_name(gint32 image_id
+                       ,gint32 *active_layer
+                       ,gint32 *stack_pos
+                       )
+{
+  gchar      *layer_name;
+  gint32     *l_layers_list;
+  gint        l_nlayers;
+  gint        l_idx;
+  gint        l_pos;
+  gboolean    l_is_onion;
+  gint32      l_layer_id;
+  
+  layer_name = NULL;
+
+  *stack_pos = -1;
+  *active_layer = gimp_image_get_active_layer(image_id);
+  
+  if(*active_layer >= 0)
+  {
+    layer_name = gimp_drawable_get_name(*active_layer);
+
+    /* findout stackposition of the active layer
+     * (ignoring onionskin layer positions)
+     */
+    l_layers_list = gimp_image_get_layers(image_id, &l_nlayers);
+    if(l_layers_list)
+    {
+      l_pos = 0;
+      for(l_idx=0;l_idx < l_nlayers;l_idx++)
+      {
+        l_layer_id = l_layers_list[l_idx];
+        if(l_layer_id == *active_layer)
+        {
+          *stack_pos = l_pos;
+          break;
+        }
+        
+        l_is_onion = gap_onion_base_check_is_onion_layer(l_layer_id);
+        if(!l_is_onion)
+        {
+          l_pos++;
+        }
+      }
+    
+      g_free(l_layers_list);
+    }
+  }
+  
+  return (layer_name);
+}  /* end p_get_active_layer_name */
+
+
+/* ---------------------------------
+ * p_do_active_layer_tracking
+ * ---------------------------------
+ * set the active layer in the specified image
+ * due to settings of active layer tracking
+ */
+void
+p_do_active_layer_tracking(gint32 image_id
+                       ,GapVinVideoInfo *vin_ptr
+                       ,gchar *ref_layer_name
+                       ,gint32 ref_layer_stackpos
+                       )
+{
+  switch(vin_ptr->active_layer_tracking)
+  {
+    case GAP_ACTIVE_LAYER_TRACKING_BY_NAME:
+      p_set_active_layer_by_name(image_id, ref_layer_name, ref_layer_stackpos);
+      break;
+    case GAP_ACTIVE_LAYER_TRACKING_BY_STACKPOS:
+      p_set_active_layer_by_pos(image_id, ref_layer_stackpos);
+      break;
+    default: /* GAP_ACTIVE_LAYER_TRACKING_OFF */
+      break;
+  }
+}  /* end p_do_active_layer_tracking */
+
+
+/* -----------------------------
+ * gap_lib_shorten_filename
+ * -----------------------------
+ * resulting string is built from prefix filename  and suffix
+ *    filename will be shortened when
+ *    prefix + " " + filename + " " + suffix 
+ *    is longer then max_chars.
+ * examples:
+ *    gap_lib_shorten_filenam("prefix", "this_is_a_very_long_filename", NULL, 20)
+ *    returns:  "prefix ...g_filename"
+ *
+ *    gap_lib_shorten_filenam("prefix", "shortname", NULL, 20)
+ *    returns:  "prefix shortname"
+ *
+ * the caller is responsible to g_free the returned string
+ */
+char *
+gap_lib_shorten_filename(const char *prefix
+                        ,const char *filename
+			,const char *suffix
+			,gint32 max_chars
+			)
+{
+  gint len_prefix;
+  gint len_fname;
+  const char *pfx;
+  char       *fnam;
+  char       *ret_string;
+  
+  len_prefix = 0;
+  len_fname = 0;
+  ret_string = NULL;
+  pfx = prefix;
+  if(prefix)
+  {
+    len_prefix = strlen(prefix);
+    if((len_prefix + 10) > max_chars)
+    {
+      pfx = NULL;
+      len_prefix = 0;
+    }
+    else
+    {
+      len_prefix++;  /* for the space between prefix and fname */
+    }
+  }
+
+  
+  if(filename)
+  {
+    fnam = NULL;
+    
+    if(suffix)
+    {
+      fnam = g_strdup_printf("%s %s", filename, suffix);
+    }
+    else
+    {
+      fnam = g_strdup(filename);
+    }
+
+
+    len_fname = strlen(fnam);
+    if((len_fname + len_prefix) <= max_chars)
+    {
+      if(pfx)
+      {
+        ret_string = g_strdup_printf("%s %s"
+	                            ,pfx
+				    ,fnam
+				    );
+      }
+      else
+      {
+        ret_string = g_strdup_printf("%s"
+				    ,fnam
+				    );
+      }
+    }
+    else
+    {
+      gint fname_idx;
+      gint len_rest;
+      
+      len_rest = (max_chars - len_prefix - 3);
+      fname_idx = len_fname - len_rest;
+      
+      if(pfx)
+      {
+        ret_string = g_strdup_printf("%s ...%s"
+	                            ,pfx
+				    ,&fnam[fname_idx]
+				    );
+      }
+      else
+      {
+        ret_string = g_strdup_printf("...%s"
+				    ,&fnam[fname_idx]
+				    );
+      }
+    }
+ 
+    g_free(fnam);
+    return (ret_string);
+     
+  }
+
+  ret_string = g_strdup(prefix);
+  return(ret_string);
+}  /* end gap_lib_shorten_filename */			
+
+
 /* ============================================================================
  * p_strdup_*_underscore
  *   duplicate string and if last char is no underscore add one at end.
@@ -272,7 +709,7 @@ gap_lib_searchpath_for_exefile(const char *exefile, const char *path)
 
     if(*pp == G_SEARCHPATH_SEPARATOR)
     {
-      *pp = '\0';  /* terminate dirpath string at seperator */
+      *pp = '\0';  /* terminate dirpath string at separator */
     }
 
     if (*pp == '\0')
@@ -486,14 +923,13 @@ gap_lib_alloc_basename(const char *imagename, long *number)
   l_ptr = &l_fname[strlen(l_fname)];
   if(l_ptr != l_fname) l_ptr--;
   l_idx = 1;
-  while(l_ptr != l_fname)
+  while(TRUE)
   {
     if((*l_ptr >= '0') && (*l_ptr <= '9'))
     {
       *number += ((*l_ptr - '0') * l_idx);
        l_idx = l_idx * 10;
       *l_ptr = '\0';
-       l_ptr--;
     }
     else
     {
@@ -506,6 +942,12 @@ gap_lib_alloc_basename(const char *imagename, long *number)
        */
        break;
     }
+    if(l_ptr == l_fname)
+    {
+      break;
+    }
+
+    l_ptr--;
   }
 
   if(gap_debug) printf("DEBUG gap_lib_alloc_basename  result:'%s'\n", l_fname);
@@ -574,6 +1016,10 @@ gap_lib_alloc_fname_fixed_digits(char *basename, long nr, char *extension, long
 
   switch(digits)
   {
+    case 8:  l_fname = g_strdup_printf("%s%08ld%s", basename, nr, extension);
+             break;
+    case 7:  l_fname = g_strdup_printf("%s%07ld%s", basename, nr, extension);
+             break;
     case 6:  l_fname = g_strdup_printf("%s%06ld%s", basename, nr, extension);
              break;
     case 5:  l_fname = g_strdup_printf("%s%05ld%s", basename, nr, extension);
@@ -607,13 +1053,13 @@ gap_lib_alloc_fname(char *basename, long nr, char *extension)
   {
     const char   *l_env;
 
-    default_digits = 6;
+    default_digits = GAP_LIB_DEFAULT_DIGITS;
 
     l_env = g_getenv("GAP_FRAME_DIGITS");
     if(l_env != NULL)
     {
       default_digits = atol(l_env);
-      default_digits = CLAMP(default_digits, 1 , 6);
+      default_digits = CLAMP(default_digits, 1 , GAP_LIB_MAX_DIGITS);
     }
   }
 
@@ -657,7 +1103,7 @@ gap_lib_alloc_fname6(char *basename, long nr, char *extension, long default_digi
   l_fname = (char *)g_malloc(l_len);
 
     l_digits_used = default_digits;
-    if(nr < 100000)
+    if(nr < 10000000)
     {
        /* try to figure out if the frame numbers are in
         * 6-digit style, with leading zeroes  "frame_000001.xcf"
@@ -668,7 +1114,9 @@ gap_lib_alloc_fname6(char *basename, long nr, char *extension, long default_digi
 
        while(l_nr_chk >= 0)
        {
-         /* check if frame is on disk with 6-digit style framenumber */
+         /* check if frame is on disk with 6-digit style framenumber
+	  * (we check 6-digit style first because this is the GAP default style)
+	  */
          g_snprintf(l_fname, l_len, "%s%06ld%s", basename, l_nr_chk, extension);
          if (gap_lib_file_exists(l_fname))
          {
@@ -676,6 +1124,25 @@ gap_lib_alloc_fname6(char *basename, long nr, char *extension, long default_digi
             break;
          }
 
+         /* check if frame is on disk with 8-digit style framenumber */
+         g_snprintf(l_fname, l_len, "%s%08ld%s", basename, l_nr_chk, extension);
+         if (gap_lib_file_exists(l_fname))
+         {
+            l_digits_used = 8;
+            break;
+         }
+
+         /* check if frame is on disk with 7-digit style framenumber */
+         g_snprintf(l_fname, l_len, "%s%07ld%s", basename, l_nr_chk, extension);
+         if (gap_lib_file_exists(l_fname))
+         {
+            l_digits_used = 7;
+            break;
+         }
+	 
+
+
+	 
          /* check if frame is on disk with 4-digit style framenumber */
          g_snprintf(l_fname, l_len, "%s%04ld%s", basename, l_nr_chk, extension);
          if (gap_lib_file_exists(l_fname))
@@ -731,7 +1198,7 @@ gap_lib_alloc_fname6(char *basename, long nr, char *extension, long default_digi
     }
     else
     {
-      /* numbers > 100000 have 6 digits or more */
+      /* numbers > 10000000 have 9 digits or more */
       l_digits_used = 0;
     }
 
@@ -741,6 +1208,10 @@ gap_lib_alloc_fname6(char *basename, long nr, char *extension, long default_digi
   {
     case 6:  l_fname = g_strdup_printf("%s%06ld%s", basename, nr, extension);
              break;
+    case 8:  l_fname = g_strdup_printf("%s%08ld%s", basename, nr, extension);
+             break;
+    case 7:  l_fname = g_strdup_printf("%s%07ld%s", basename, nr, extension);
+             break;
     case 5:  l_fname = g_strdup_printf("%s%05ld%s", basename, nr, extension);
              break;
     case 4:  l_fname = g_strdup_printf("%s%04ld%s", basename, nr, extension);
@@ -777,7 +1248,7 @@ gap_lib_exists_frame_nr(GapAnimInfo *ainfo_ptr, long nr, long *l_has_digits)
   l_len = (strlen(ainfo_ptr->basename)  + strlen(ainfo_ptr->extension) + 10);
   l_fname = (char *)g_malloc(l_len);
 
-  l_digits_used = 6;
+  l_digits_used = GAP_LIB_DEFAULT_DIGITS;
   l_nr_chk = nr;
 
   while(l_nr_chk >= 0)
@@ -794,6 +1265,35 @@ gap_lib_exists_frame_nr(GapAnimInfo *ainfo_ptr, long nr, long *l_has_digits)
         break;
      }
 
+     /* check if frame is on disk with 8-digit style framenumber */
+     g_snprintf(l_fname, l_len, "%s%08ld%s", ainfo_ptr->basename, l_nr_chk, ainfo_ptr->extension);
+     if (gap_lib_file_exists(l_fname))
+     {
+        l_digits_used = 8;
+        if(l_nr_chk == nr)
+        {
+          l_exists = TRUE;
+        }
+        break;
+     }
+
+     /* check if frame is on disk with 7-digit style framenumber */
+     g_snprintf(l_fname, l_len, "%s%07ld%s", ainfo_ptr->basename, l_nr_chk, ainfo_ptr->extension);
+     if (gap_lib_file_exists(l_fname))
+     {
+        l_digits_used = 7;
+        if(l_nr_chk == nr)
+        {
+          l_exists = TRUE;
+        }
+        break;
+     }
+     
+     
+     
+     
+     
+     
      /* check if frame is on disk with 4-digit style framenumber */
      g_snprintf(l_fname, l_len, "%s%04ld%s", ainfo_ptr->basename, l_nr_chk, ainfo_ptr->extension);
      if (gap_lib_file_exists(l_fname))
@@ -1206,7 +1706,7 @@ gap_lib_get_frame_nr(gint32 image_id)
  * (using save as)
  * while the gap-plugin's dialog was open.
  * return: 0 .. OK
- *        -1 .. Changed (or error occured)
+ *        -1 .. Changed (or error occurred)
  * ============================================================================
  */
 int
@@ -1734,7 +2234,7 @@ p_save_old_frame(GapAnimInfo *ainfo_ptr, GapVinVideoInfo *vin_ptr)
    */
   if(gap_lib_gap_check_save_needed(ainfo_ptr->image_id))
   {
-    /* check and peroform automatic onionskinlayer remove */
+    /* check and perform automatic onionskinlayer remove */
     if(vin_ptr)
     {
       if((vin_ptr->auto_delete_before_save) && (vin_ptr->onionskin_auto_enable))
@@ -1835,6 +2335,9 @@ gap_lib_load_named_frame (gint32 old_image_id, char *lod_name)
       /* delete the old image */
       gimp_image_delete(old_image_id);
 
+
+      gimp_image_undo_disable(l_new_image_id);
+
       /* use the original lod_name */
       gimp_image_set_filename (l_new_image_id, lod_name);
 
@@ -1846,6 +2349,7 @@ gap_lib_load_named_frame (gint32 old_image_id, char *lod_name)
        */
       gap_navat_update_active_image(old_image_id, l_new_image_id);
 
+      gimp_image_undo_enable(l_new_image_id);
 
       return l_new_image_id;
    }
@@ -1861,6 +2365,8 @@ gap_lib_load_named_frame (gint32 old_image_id, char *lod_name)
  *
  * make new_filename of next file to load, check if that file does exist on disk
  * then save current image and replace it, by loading the new_filename
+ * this includes automatic onionskin layer handling and
+ * active layer tracking
  *
  * return image_id (of the new loaded frame) on success
  *        or -1 on errors
@@ -1872,8 +2378,17 @@ gap_lib_replace_image(GapAnimInfo *ainfo_ptr)
   gint32 image_id;
   GapVinVideoInfo *vin_ptr;
   gboolean  do_onionskin_crate;
+  gint32    ref_active_layer;
+  gint32    ref_layer_stackpos;
+  gchar    *ref_layer_name;
 
   do_onionskin_crate  = FALSE;
+
+  ref_layer_name = p_get_active_layer_name(ainfo_ptr->image_id
+                                          ,&ref_active_layer
+                                          ,&ref_layer_stackpos
+                                          );  
+  
   if(ainfo_ptr->new_filename != NULL) g_free(ainfo_ptr->new_filename);
   ainfo_ptr->new_filename = gap_lib_alloc_fname(ainfo_ptr->basename,
                                       ainfo_ptr->frame_nr,
@@ -1932,13 +2447,27 @@ gap_lib_replace_image(GapAnimInfo *ainfo_ptr)
              , FALSE                   /* use_cache */
              );
     }
+
+    p_do_active_layer_tracking(image_id
+                       ,vin_ptr
+                       ,ref_layer_name
+                       ,ref_layer_stackpos
+                       );
+
     g_free(vin_ptr);
   }
 
+  if(ref_layer_name)
+  {
+    g_free(ref_layer_name);
+  }
+
   return(image_id);
 }	/* end gap_lib_replace_image */
 
 
+
+
 /* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */
 /* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */
 /* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */
@@ -2327,7 +2856,7 @@ gap_vid_edit_paste(GimpRunMode run_mode, gint32 image_id, long paste_mode)
 
            /* convert tmp image to dest type */
            case GIMP_INDEXED:
-             l_cmap = gimp_image_get_cmap(image_id, &l_ncolors);
+             l_cmap = gimp_image_get_colormap(image_id, &l_ncolors);
              if(gap_debug) printf("DEBUG: convert to INDEXED %d colors\n", (int)l_ncolors);
 
              l_palette_filename = g_strdup_printf("%s%spalettes%s%s"
@@ -2431,3 +2960,68 @@ gap_vid_edit_paste(GimpRunMode run_mode, gint32 image_id, long paste_mode)
 
   return(rc);
 }       /* end gap_vid_edit_paste */
+
+
+/* --------------------------------
+ * gap_lib_fprintf_gdouble
+ * --------------------------------
+ * print prefix and gdouble value to file
+ * (always use "." as decimalpoint, independent of LOCALE language settings)
+ */
+void
+gap_lib_fprintf_gdouble(FILE *fp, gdouble value, gint digits, gint precision_digits, const char *pfx)
+{
+  gchar l_dbl_str[G_ASCII_DTOSTR_BUF_SIZE];
+  gchar l_fmt_str[20];
+  gint  l_len;
+
+  g_snprintf(l_fmt_str, sizeof(l_fmt_str), "%%.%df", (int)precision_digits);
+  g_ascii_formatd(l_dbl_str, sizeof(l_dbl_str), l_fmt_str, value);
+
+  fprintf(fp, "%s", pfx);
+
+  /* make leading blanks */
+  l_len = strlen(l_dbl_str) - (digits + 1 +  precision_digits);
+  while(l_len < 0)
+  {
+    fprintf(fp, " ");
+    l_len++;
+  }
+  fprintf(fp, "%s", l_dbl_str);
+}  /* end gap_lib_fprintf_gdouble */
+
+
+/* ============================================================================
+ * gap_lib_sscan_flt_numbers
+ * ============================================================================
+ * scan the blank separated buffer for 2 integer and 13 float numbers.
+ * always use "." as decimalpoint in the float numbers regardless to LANGUAGE settings
+ * return a counter that tells how many numbers were scanned successfully
+ */
+gint
+gap_lib_sscan_flt_numbers(gchar   *buf
+                  , gdouble *farr
+		  , gint     farr_max
+		  )
+{
+  gint  l_cnt;
+  gchar *nptr;
+  gchar *endptr;
+
+  l_cnt =0;
+  nptr  = buf;
+  while(l_cnt < farr_max)
+  {
+    endptr = nptr;
+    farr[l_cnt] = g_ascii_strtod(nptr, &endptr);
+    if(nptr == endptr)
+    {
+      break;  /* pointer was not advanced because no valid floatnumber was scanned */
+    }
+    nptr = endptr;
+
+    l_cnt++;  /* count successful scans */
+  }
+
+  return (l_cnt);
+}  /* end gap_lib_sscan_flt_numbers */
