@@ -28,6 +28,9 @@
  */
 
 /* revision history:
+ * gimp   2.1.0b;    2004/10/17  hof: bugfix Function: Add layermask from selection
+ *                                    optim: direct processing for the current image
+ * gimp   2.1.0b;    2004/08/11  hof: new action_modes "Copy layermask from layer above/below"
  * gimp   1.3.25a;   2004/01/21  hof: message text fixes (# 132030)
  * gimp   1.3.24a;   2004/01/17  hof: added Layermask Handling
  * gimp   1.3.20d;   2003/09/20  hof: sourcecode cleanup,
@@ -73,226 +76,13 @@
 #include "gap_image.h"
 #include "gap_range_ops.h"
 #include "gap_mod_layer.h"
+#include "gap_mod_layer_dialog.h"
 
 
 extern      int gap_debug; /* ==0  ... dont print debug infos */
 
 
-/* ============================================================================
- * p_layer_modify_dialog
- *   retcode
- *               0   ... Generate Paramfile
- *               1   ... Generate Paramfile and start mpeg_encode
- * ============================================================================
- */
-static
-int p_layer_modify_dialog(GapAnimInfo *ainfo_ptr,
-                   gint32 *range_from,  gint32 *range_to,
-                   gint32 *action_mode, gint32 *sel_mode,
-                   gint32 *sel_case,    gint32 *sel_invert,
-                   char *sel_pattern,   char   *new_layername)
-{
-  static GapArrArg  argv[9];
-  static GapArrButtonArg  b_argv[2];
-  gint   l_rc;
-
-  /* Layer select modes */
-  static char *sel_args[7]    = { N_("Pattern is equal to layer name"),
-                                  N_("Pattern is start of layer name"),
-                                  N_("Pattern is end of layer name"),
-                                  N_("Pattern is a part of layer name"),
-                                  N_("Pattern is a list of layerstack numbers"),
-                                  N_("Pattern is a list of reverse layerstack numbers"),
-                                  N_("All visible (ignore pattern)")
-                                  };
-  static char *sel_help[7]    = { N_("Select all layers where layer name is equal to pattern"),
-                                  N_("Select all layers where layer name starts with pattern"),
-                                  N_("Select all layers where layer name ends up with pattern"),
-                                  N_("Select all layers where layer name contains pattern"),
-                                  N_("Select layerstack positions where 0 is the top layer.\nExample: 0, 4-5, 8"),
-                                  N_("Select layerstack positions where 0 is the background layer.\nExample: 0, 4-5, 8"),
-                                  N_("Select all visible layers")
-                                  };
-
-  /* action items what to do with the selected layer(s) */
-  static char *action_args[32]= { N_("Set layer(s) visible"),
-                                  N_("Set layer(s) invisible"),
-                                  N_("Set layer(s) linked"),
-                                  N_("Set layer(s) unlinked"),
-                                  N_("Raise layer(s)"),
-                                  N_("Lower layer(s)"),
-                                  N_("Merge layer(s) expand as necessary"),
-                                  N_("Merge layer(s) clipped to image"),
-                                  N_("Merge layer(s) clipped to bg-layer"),
-                                  N_("Apply filter on layer(s)"),
-                                  N_("Duplicate layer(s)"),
-                                  N_("Delete layer(s)"),
-                                  N_("Rename layer(s)"),
-                                  N_("Replace selection"),
-                                  N_("Add selection"),
-                                  N_("Subtract selection"),
-                                  N_("Intersect selection"),
-                                  N_("Selection none"),
-                                  N_("Selection all"),
-                                  N_("Selection invert"),
-                                  N_("Save selection to channel"),
-				  N_("Load selection from channel"),
-                                  N_("Delete channel (by name)"),
-				  N_("Add alpha channel"),
-				  N_("Add white layermask (opaque)"),
-				  N_("Add black layermask (transparent)"),
-				  N_("Add layermask from alpha"),
-				  N_("Add layermask transfer from alpha"),
-				  N_("Add layermask from selection"),
-				  N_("Add layermask from bw copy"),
-				  N_("Delete layermask"),
-				  N_("Apply layermask")
-                                  };
-
-
-/*
-  static char *action_help[32]  = {"set all selected layers visible",
-                                  "set all selected layers  invisible",
-                                  "set all selected layers  linked",
-                                  "set all selected layers  unlinked",
-                                  "raise all selected layers",
-                                  "lower all selected layers",
-                                  "merge expand as necessary",
-                                  "merge clipped to image",
-                                  "merge clipped to bg-layer",
-                                  "APPLY FILTER to all selected layers",
-                                  "duplicate all selected layers",
-                                  "delete REMOVES all selected layers",
-                                  "rename all selected layers\nto NewLayername.",
-
-				  "Replace Selection by Selection of the invoking Frame Image",
-				  "Add Selection",
-				  "Subtract Selection",
-				  "Intersect Selection",
-				  "Clear Selection in all handled Frames",
-				  "Select All in all handled Frames",
-				  "Invert Selection in all handled Frames",
-				  "Save Selection as Channel",
-				  "Load Selection from Channel",
-				  "Delete Channel by Name"
-                                  };
-*/
-
-  guint get_textize_loop;
-
-  for (get_textize_loop = 0; get_textize_loop < G_N_ELEMENTS (sel_args) ; get_textize_loop++) {
-    sel_args[get_textize_loop] = gettext(sel_args[get_textize_loop]);
-    sel_help[get_textize_loop] = gettext(sel_help[get_textize_loop]);
-  }
-  for (get_textize_loop = 0; get_textize_loop < G_N_ELEMENTS (action_args); get_textize_loop++)
-    action_args[get_textize_loop] = gettext(action_args[get_textize_loop]);
-
-  l_rc = -1;
-
-  /* the 2 Action Buttons */
-    b_argv[0].but_txt  = GTK_STOCK_CANCEL;
-    b_argv[0].but_val  = -1;
-    b_argv[1].but_txt  = GTK_STOCK_OK;
-    b_argv[1].but_val  = 0;
-
-  gap_arr_arg_init(&argv[0], GAP_ARR_WGT_LABEL);
-  argv[0].label_txt = _("Perform function on one or more layer(s)\n"
-                        "in all frames of the selected frame range\n");
-
-  gap_arr_arg_init(&argv[1], GAP_ARR_WGT_INT_PAIR);
-  argv[1].constraint = TRUE;
-  argv[1].label_txt = _("From Frame:");
-  argv[1].help_txt  = _("First handled frame");
-  argv[1].int_min   = (gint)ainfo_ptr->first_frame_nr;
-  argv[1].int_max   = (gint)ainfo_ptr->last_frame_nr;
-  argv[1].int_ret   = (gint)ainfo_ptr->curr_frame_nr;
-
-  gap_arr_arg_init(&argv[2], GAP_ARR_WGT_INT_PAIR);
-  argv[2].constraint = TRUE;
-  argv[2].label_txt = _("To Frame:");
-  argv[2].help_txt  = _("Last handled frame");
-  argv[2].int_min   = (gint)ainfo_ptr->first_frame_nr;
-  argv[2].int_max   = (gint)ainfo_ptr->last_frame_nr;
-  argv[2].int_ret   = (gint)ainfo_ptr->last_frame_nr;
-
-
-  /* Layer select mode RADIO buttons */
-  gap_arr_arg_init(&argv[3], GAP_ARR_WGT_RADIO);
-  argv[3].label_txt = _("Layer Selection:");
-  argv[3].radio_argc = G_N_ELEMENTS (sel_args);
-  argv[3].radio_argv = sel_args;
-  argv[3].radio_help_argv = sel_help;
-  argv[3].radio_ret  = 4;
-
-  /* Layer select pattern string */
-  sel_pattern[0] = '0';
-  sel_pattern[1] = '\0';
-  gap_arr_arg_init(&argv[4], GAP_ARR_WGT_TEXT);
-  argv[4].label_txt = _("Layer Pattern:");
-  argv[4].entry_width = 140;       /* pixel */
-  argv[4].help_txt  = _("String to identify layer names or layerstack position numbers. Example: 0,3-5");
-  argv[4].text_buf_len = MAX_LAYERNAME;
-  argv[4].text_buf_ret = sel_pattern;
-
-  /* case sensitive checkbutton */
-  gap_arr_arg_init(&argv[5], GAP_ARR_WGT_TOGGLE);
-  argv[5].label_txt = _("Case sensitive");
-  argv[5].help_txt  = _("Lowercase and uppercase letters are considered as different");
-  argv[5].int_ret   = 1;
-
-  /* invert selection checkbutton */
-  gap_arr_arg_init(&argv[6], GAP_ARR_WGT_TOGGLE);
-  argv[6].label_txt = _("Invert Layer Selection");
-  argv[6].help_txt  = _("Perform actions on all unselected layers");
-  argv[6].int_ret   = 0;
-
-  /* desired action to perform OPTIONMENU  */
-  gap_arr_arg_init(&argv[7], GAP_ARR_WGT_OPTIONMENU);
-  argv[7].label_txt = _("Function:");
-  argv[7].radio_argc = G_N_ELEMENTS (action_args);
-  argv[7].radio_argv = action_args;
-  /* argv[7].radio_help_argv = action_help */
-  argv[7].help_txt = _("Function to be performed on all selected layers");
-  argv[7].radio_ret  = 0;
-
-  /* a new name for the handled Layer(s) */
-  *new_layername = '\0';
-  gap_arr_arg_init(&argv[8], GAP_ARR_WGT_TEXT);
-  argv[8].label_txt = _("Layer Name\n or Channel Name:");
-  argv[8].entry_width = 140;       /* pixel */
-  argv[8].help_txt  = _("Layer (or channel) name for all handled layers (or channels),\n"
-                        "where the string '[######]' is replaced by the frame number.\n"
-			"This name is only relevant for the functions:\n"
-			" Duplicate layer(s)\n"
-			" Rename layer(s)\n"
-			" Save selection to channel\n"
-			" Load selection from channel\n"
-			" Delete channel (by name)"
-			);
-  argv[8].text_buf_len = MAX_LAYERNAME;
-  argv[8].text_buf_ret = new_layername;
-
-
-  l_rc =  gap_arr_std_dialog( _("Frames Modify"),
-                                _("Settings"),
-                                 9,   argv,      /* widget array */
-                                 2,   b_argv,    /* button array */
-                                 0);
-
-  /* return the entered values */
-  *range_from           = argv[1].int_ret;
-  *range_to             = argv[2].int_ret;
-  *sel_mode             = argv[3].int_ret;
-                       /*     [4] sel_pattern  */
-  *sel_case             = argv[5].int_ret;
-  *sel_invert           = argv[6].int_ret;
-  *action_mode          = argv[7].int_ret;
-                       /*     [8] l_new_layername */
-
-
-  return (l_rc);
-}	/* end p_layer_modify_dialog */
-
+#define GAP_DB_BROWSER_MODFRAMES_HELP_ID  "gap-modframes-db-browser"
 
 /* ============================================================================
  * p_pitstop_dialog
@@ -349,6 +139,50 @@ p_pitstop_dialog(gint text_flag, char *filter_procname)
 }	/* end p_pitstop_dialog */
 
 
+/* ---------------------------------
+ * p_get_nb_layer_id
+ * ---------------------------------
+ * get layer_id of the neigbour layer /above or below)
+ * the specified ref_layer
+ * with the nb_ref parameter of -1 the neigbour layer one above is picked
+ *                            , +1 selects the neigbour one below
+ * return -1 if there is no such a neigbourlayer
+ */
+gint32
+p_get_nb_layer_id(gint32 image_id, gint32 ref_layer_id, gint32 nb_ref)
+{
+  gint          l_nlayers;
+  gint32       *l_layers_list;
+  gint32        l_nb_layer_id;
+  gint32        l_idx;
+  gint32        l_nb_idx;
+
+  l_nb_layer_id = -1;
+  
+  l_layers_list = gimp_image_get_layers(image_id, &l_nlayers);
+  if(l_layers_list != NULL)
+  {
+    for(l_idx = 0; l_idx < l_nlayers; l_idx++)
+    {
+      if(l_layers_list[l_idx] == ref_layer_id)
+      {
+        l_nb_idx = l_idx + nb_ref;
+	if((l_nb_idx >= 0) && (l_nb_idx < l_nlayers))
+	{
+	  l_nb_layer_id = l_layers_list[l_nb_idx];
+	}
+        break;
+      }
+    }
+  
+    g_free (l_layers_list);
+  }
+
+  return (l_nb_layer_id);
+}  /* end p_get_nb_layer_id */
+
+
+
 /* ============================================================================
  * gap_mod_get_1st_selected
  *   return index of the 1.st selected layer
@@ -444,27 +278,35 @@ gap_mod_alloc_layli(gint32 image_id, gint32 *l_sel_cnt, gint *nlayers,
 static void
 p_raise_layer (gint32 image_id, gint32 layer_id, GapModLayliElem * layli_ptr, gint nlayers)
 {
-  if(! gimp_drawable_has_alpha (layer_id)) return; /* has no alpha channel */
-
   if(layli_ptr[0].layer_id == layer_id)  return;   /* is already on top */
 
+  if(! gimp_drawable_has_alpha (layer_id))
+  {
+    /* implicite add an alpha channel before we try to raise */
+    gimp_layer_add_alpha(layer_id);
+  }
   gimp_image_raise_layer(image_id, layer_id);
 }	/* end p_raise_layer */
 
 static void
 p_lower_layer (gint32 image_id, gint32 layer_id, GapModLayliElem * layli_ptr, gint nlayers)
 {
-  if(! gimp_drawable_has_alpha (layer_id)) return; /* has no alpha channel */
-
   if(layli_ptr[nlayers-1].layer_id == layer_id)  return;   /* is already on bottom */
 
+  if(! gimp_drawable_has_alpha (layer_id))
+  {
+    /* implicite add an alpha channel before we try to lower */
+    gimp_layer_add_alpha(layer_id);
+  }
+
   if(nlayers > 1)
   {
     if((layli_ptr[nlayers-2].layer_id == layer_id)
     && (! gimp_drawable_has_alpha (layli_ptr[nlayers-1].layer_id)))
     {
       /* the layer is one step above a "bottom-layer without alpha" */
-      return;
+      /* implicite add an alpha channel before we try to lower */
+      gimp_layer_add_alpha(layli_ptr[nlayers-1].layer_id);
     }
   }
 
@@ -825,6 +667,7 @@ p_apply_action(gint32 image_id,
 	    gimp_layer_remove_mask (l_layer_id, GIMP_MASK_APPLY);
 	  }
 	  l_layermask_id = gimp_layer_create_mask(l_layer_id, GIMP_ADD_SELECTION_MASK);
+	  gimp_layer_add_mask(l_layer_id, l_layermask_id);
 	  break;
         case GAP_MOD_ACM_LMASK_BWCOPY:
 	  if(gimp_layer_get_mask(l_layer_id) >= 0)
@@ -846,6 +689,63 @@ p_apply_action(gint32 image_id,
 	    gimp_layer_remove_mask (l_layer_id, GIMP_MASK_APPLY);
 	  }
 	  break;
+        case GAP_MOD_ACM_LMASK_COPY_FROM_UPPER_LMASK:
+        case GAP_MOD_ACM_LMASK_COPY_FROM_LOWER_LMASK:
+	  /* create black mask per default */
+	  if(gimp_layer_get_mask(l_layer_id) >= 0)
+	  {
+	    gimp_layer_remove_mask (l_layer_id, GIMP_MASK_APPLY);
+	  }
+	  l_layermask_id = gimp_layer_create_mask(l_layer_id, GIMP_ADD_BLACK_MASK);
+	  gimp_layer_add_mask(l_layer_id, l_layermask_id);
+	  /* get layermask_id from upper neigbour layer (if there is any) */
+	  if(l_layermask_id >= 0)
+	  {
+	    gint32 l_nb_mask_id;
+	    gint32 l_nb_layer_id;
+	    gint32 l_nb_ref;
+	    gint32 l_fsel_layer_id;
+	    gint32 l_nb_had_layermask;
+	    
+	    l_nb_ref = 1;
+            if(action_mode == GAP_MOD_ACM_LMASK_COPY_FROM_UPPER_LMASK)
+	    {
+	      l_nb_ref = -1;
+	    }
+	    l_nb_layer_id = p_get_nb_layer_id(image_id, l_layer_id, l_nb_ref);
+	    if(l_nb_layer_id >= 0)
+	    {
+	      l_nb_mask_id = gimp_layer_get_mask(l_nb_layer_id);
+	      l_nb_had_layermask = TRUE;
+	      if(l_nb_mask_id < 0)
+	      {
+	        /* the referenced neigbour layer had no layermask
+		 * in this case we create one as bw-copy temporary
+		 */
+	        l_nb_had_layermask = FALSE;
+		l_nb_mask_id = gimp_layer_create_mask(l_nb_layer_id, GIMP_ADD_COPY_MASK);
+	        gimp_layer_add_mask(l_nb_layer_id, l_nb_mask_id);
+	      }
+	      
+	      if(l_nb_mask_id >= 0)
+	      {
+	        /* the referenced neigbour layer has a layermask
+		 * we can copy from this mask now
+		 */
+		gimp_selection_none(image_id);
+		gimp_edit_copy(l_nb_mask_id);
+		l_fsel_layer_id = gimp_edit_paste(l_layermask_id, FALSE);
+		gimp_floating_sel_anchor(l_fsel_layer_id);
+		
+		if(l_nb_had_layermask == FALSE)
+		{
+		  /* remove the temporary created layermask in the neigbour layer */
+		  gimp_layer_remove_mask (l_nb_layer_id, GIMP_MASK_DISCARD);
+		}
+	      }
+	    }
+	  }
+	  break;
         default:
 	  break;
       }
@@ -888,7 +788,8 @@ p_do_filter_dialogs(GapAnimInfo *ainfo_ptr,
                             gap_filt_pdb_constraint_proc_sel1,
                             gap_filt_pdb_constraint_proc_sel2,
                             &l_browser_result,
-			    image_id) < 0)
+			    image_id,
+			    GAP_DB_BROWSER_MODFRAMES_HELP_ID) < 0)
   {
       if(gap_debug) fprintf(stderr, "DEBUG: gap_db_browser_dialog cancelled\n");
       return -1;
@@ -1094,6 +995,7 @@ p_frames_modify(GapAnimInfo *ainfo_ptr,
   GapFiltPdbApplyMode  l_apply_mode;
   char         *l_last_frame_filename;
   gint          l_count;
+  gboolean      l_operating_on_current_image;
 
 
 
@@ -1107,7 +1009,7 @@ p_frames_modify(GapAnimInfo *ainfo_ptr,
     gimp_progress_init( _("Modifying frames/layer(s)..."));
   }
 
-
+  l_operating_on_current_image = TRUE;
   l_begin = range_from;
   l_end   = range_to;
   l_tmp_image_id = -1;
@@ -1164,8 +1066,18 @@ p_frames_modify(GapAnimInfo *ainfo_ptr,
     if(ainfo_ptr->new_filename == NULL)
        goto error;
 
-    /* load current frame into temporary image */
-    l_tmp_image_id = gap_lib_load_image(ainfo_ptr->new_filename);
+    if(ainfo_ptr->curr_frame_nr == l_cur_frame_nr)
+    {
+      /* directly operate on the current image (where we were invoked from) */
+      l_operating_on_current_image = TRUE;
+      l_tmp_image_id = ainfo_ptr->image_id;
+    }
+    else
+    {
+      /* load current frame into temporary image */
+      l_operating_on_current_image = FALSE;
+      l_tmp_image_id = gap_lib_load_image(ainfo_ptr->new_filename);
+    }
     if(l_tmp_image_id < 0)
        goto error;
 
@@ -1332,8 +1244,11 @@ p_frames_modify(GapAnimInfo *ainfo_ptr,
       l_dpy_id = -1;
     }
 
-    /* destroy the tmp image */
-    gimp_image_delete(l_tmp_image_id);
+    if(l_operating_on_current_image == FALSE)
+    {
+      /* destroy the tmp image */
+      gimp_image_delete(l_tmp_image_id);
+    }
 
     if(l_rc != 0)
       goto error;
@@ -1365,7 +1280,10 @@ error:
       gimp_display_delete(l_dpy_id);
       l_dpy_id = -1;
   }
-  if(l_tmp_image_id >= 0) gimp_image_delete(l_tmp_image_id);
+  if((l_tmp_image_id >= 0) && (l_operating_on_current_image == FALSE))
+  {
+    gimp_image_delete(l_tmp_image_id);
+  }
   if(l_layli_ptr != NULL) g_free(l_layli_ptr);
   if(l_plugin_iterator != NULL)  g_free(l_plugin_iterator);
   return -1;
@@ -1407,11 +1325,10 @@ gint gap_mod_layer(GimpRunMode run_mode, gint32 image_id,
     {
       if(run_mode == GIMP_RUN_INTERACTIVE)
       {
-         l_rc = p_layer_modify_dialog (ainfo_ptr, &l_from, &l_to,
+         l_rc = gap_mod_frames_dialog (ainfo_ptr, &l_from, &l_to,
 	                               &l_action_mode,
 				       &l_sel_mode, &sel_case, &sel_invert,
 				       &l_sel_pattern[0], &l_new_layername[0]);
-
       }
       else
       {
@@ -1430,22 +1347,19 @@ gint gap_mod_layer(GimpRunMode run_mode, gint32 image_id,
 
       if(l_rc >= 0)
       {
-         if(gap_lib_gap_check_save_needed(ainfo_ptr->image_id))
-         {
-           l_rc = gap_lib_save_named_frame(ainfo_ptr->image_id, ainfo_ptr->old_filename);
-         }
-         if(l_rc >= 0)
-         {
+        /* no need to save the current image before processing
+	 * because the p_frames_modify procedure operates directly on the current frame
+	 * and loads all other frames from disc.
+	 * futher all successful processed frames are saved back to disk
+	 * (including the current one)
+	 */
            l_rc = p_frames_modify(ainfo_ptr, l_from, l_to,
 	                          l_action_mode,
 				  l_sel_mode, sel_case, sel_invert,
 				  &l_sel_pattern[0], &l_new_layername[0]
 				 );
-           gap_lib_load_named_frame(ainfo_ptr->image_id, ainfo_ptr->old_filename);
-         }
       }
 
-
     }
     gap_lib_free_ainfo(&ainfo_ptr);
   }
