@@ -3,8 +3,6 @@
  *
  *  Wrapper Plugin for GIMP Curves tool
  *
- *  INSTALL: gimptool-1.3 --install wr_color_curve.c
- *
  * Warning: This is just a QUICK HACK to enable
  *          Animated Filterapply in conjunction with the
  *          GIMP Curve Tool.
@@ -39,6 +37,7 @@
  */
 
 /* Revision history
+ *  (2005/01/29)  v2.1   hof: added Help
  *  (2004/01/15)  v1.3   hof: integrated into GIMP-GAP
  *  (2003/10/30)  v1.3   hof: adapted for gimp-1.3.x and gtk+2.2 API
  *  (2002/10/27)  v1.03  hof: appear in menu (enable filtermacro use)
@@ -59,6 +58,8 @@
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
+#include "gap_stock.h"
+
 #include "gap-intl.h"
 
 
@@ -74,6 +75,7 @@
 #define PLUG_IN_DATA_ITER_FROM  "plug_in_wr_curves_ITER_FROM"
 #define PLUG_IN_DATA_ITER_TO    "plug_in_wr_curves_ITER_TO"
 
+#define PLUG_IN_HELP_ID         "plug-in-wr-curves"
 
 /* pointval_t
  *  wanted to use gdouble to store plugin data
@@ -775,28 +777,33 @@ p_filesel_ok_callback (GtkWidget *widget,
 
 static void
 wr_curve_load_callback (GtkWidget *w,
-		      gpointer   data)
+		      WrCurveDialog *wcd)
 {
-  WrCurveDialog *wcd;
   GtkWidget *filesel;
 
-  wcd = (WrCurveDialog *) data;
+  if(wcd == NULL)
+  {
+    return;
+  }
   if(wcd->filesel != NULL)
+  {
+     gtk_window_present(GTK_WINDOW(wcd->filesel));
      return;   /* filesel is already open */
-
+  }
+  
   filesel = gtk_file_selection_new ( _("Load color curve from file"));
   wcd->filesel = filesel;
 
-  gtk_window_position (GTK_WINDOW (filesel), GTK_WIN_POS_MOUSE);
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-		      "clicked", (GtkSignalFunc) p_filesel_ok_callback,
-		      wcd);
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
+  gtk_window_set_position (GTK_WINDOW (filesel), GTK_WIN_POS_MOUSE);
+  g_signal_connect (GTK_FILE_SELECTION (filesel)->ok_button,
+                    "clicked", (GtkSignalFunc) p_filesel_ok_callback,
+                    wcd);
+  g_signal_connect (GTK_FILE_SELECTION (filesel)->cancel_button,
 		      "clicked", (GtkSignalFunc) p_filesel_close_cb,
 		      wcd);
-  gtk_signal_connect (GTK_OBJECT (filesel), "destroy",
-		      (GtkSignalFunc) p_filesel_close_cb,
-		      wcd);
+  g_signal_connect (filesel, "destroy",
+                    (GtkSignalFunc) p_filesel_close_cb,
+                    wcd);
   if(wcd->filename)
   {
     gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
@@ -869,16 +876,18 @@ do_dialog (wr_curves_val_t *cuvals)
 
   /* Init UI  */
   gimp_ui_init ("wr_curves", FALSE);
+  gap_stock_init();
 
 
   /*  The curve_bend dialog  */
   wcd = g_malloc (sizeof (WrCurveDialog));
   wcd->run = FALSE;
+  wcd->filesel = NULL;
 
   /*  The dialog and main vbox  */
   dialog = gimp_dialog_new (_("CurvesFile"), "curves_wrapper",
                                NULL, 0,
-			       gimp_standard_help_func, NULL, /* "filters/color/wrapper.html" */
+			       gimp_standard_help_func, PLUG_IN_HELP_ID,
 
                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                GTK_STOCK_OK,     GTK_RESPONSE_OK,
@@ -899,12 +908,12 @@ do_dialog (wr_curves_val_t *cuvals)
   /* the hbox */
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
   gtk_widget_show (hbox);
 
   /*  The Load button  */
   button = gtk_button_new_with_label (_("Load Curve"));
-  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button), "clicked",
 		    G_CALLBACK (wr_curve_load_callback),
 		    wcd);
@@ -916,9 +925,9 @@ do_dialog (wr_curves_val_t *cuvals)
 
   /*  The filename entry */
   entry = gtk_entry_new();
-  gtk_widget_set_usize(entry, 350, 0);
+  gtk_widget_set_size_request(entry, 350, -1);
   gtk_entry_set_text(GTK_ENTRY(entry), "");
-  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(entry), "changed",
 		   G_CALLBACK (text_entry_callback),
 		   wcd);
