@@ -63,6 +63,7 @@
 #define PLUG_IN_ITER_NAME       "plug_in_wr_color_levels_Iterator"
 #define PLUG_IN_DATA_ITER_FROM  "plug_in_wr_color_levels_ITER_FROM"
 #define PLUG_IN_DATA_ITER_TO    "plug_in_wr_color_levels_ITER_TO"
+#define PLUG_IN_HELP_ID         "plug-in-wr-color-levels"
 
 
 
@@ -584,7 +585,7 @@ do_dialog (wr_levels_val_t *cuvals)
   /*  The dialog1 and main vbox  */
   dialog1 = gimp_dialog_new (_("Color Levels"), "levels_wrapper",
                                NULL, 0,
-			       gimp_standard_help_func, NULL, /* "filters/color/wrapper.html" */
+			       gimp_standard_help_func, PLUG_IN_HELP_ID, 
 
                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                GTK_STOCK_OK,     GTK_RESPONSE_OK,
@@ -609,7 +610,7 @@ do_dialog (wr_levels_val_t *cuvals)
 
 
   /* the frame */
-  frame1 = gtk_frame_new (_("Color Levels  Adjustments "));
+  frame1 = gimp_frame_new (_("Color Levels  Adjustments "));
   gtk_widget_show (frame1);
   gtk_box_pack_start (GTK_BOX (dialog_vbox1), frame1, TRUE, TRUE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (frame1), 2);
